/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.annotation.flowControl;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Controls inclusion of the element's childs (i.e. content).
 * <p/>
 * If the field/method which the annotation is applied to, returns false, then the content
 * is not output.
 * <p/>
 * It is contained within the looping constructs (such as {@link EWhile}), so the
 * condition is re-evaluated for each iteration of the loop.
 * <p/>
 * Multiple annotations are allowed for a html element. The field/method return values
 * are combined using the boolean and operation:
 * The element's content is only output if all of the fields/methods return true.
 * <p/>
 * Must be applied to a boolean field or a method which returns boolean and takes no parameters.
 * Both primitive boolean and the Boolean class is allowed.
 *
 * @see EContentIfNot
 * @see EIf
 */
@Target({ElementType.METHOD, ElementType.FIELD})
@Retention(RetentionPolicy.RUNTIME)
public @interface EContentIf
{
	/**
	 * Makes the annotation match html elements which contains the value in the 'id' attribute or as a name in the 'class' attribute.
	 * Leading underscores in the names specified in the html are ignored when comparing the names.
	 * <p/>
	 * When the parameter is specified, the 'id', 'className' or 'tag' parameters must not be specified.
	 */
	String[] value() default {};

	/**
	 * Makes the annotation match html elements which contains the value in the 'id' attribute.
	 * Leading underscores in the 'id' attribute value are ignored when comparing the names.
	 * <p/>
	 * When the parameter is specified, the 'value' parameter must not be specified. It is allowed to specify
	 * the 'className' and 'tag' parameters.
	 */
	String[] id() default {};

	/**
	 * Makes the annotation match html elements which contains the value as a name in the 'class' attribute.
	 * Leading underscores in the names specified in the html are ignored when comparing the names.
	 * <p/>
	 * When the parameter is specified, the 'value' parameter must not be specified. It is allowed to specify
	 * the 'id' and 'tag' parameters.
	 */
	String[] className() default {};

	/**
	 * Makes the annotation match html elements whose tag is the specified value.
	 * <p/>
	 * When the parameter is specified, the 'value' parameter must not be specified. It is allowed to specify
	 * the 'id' and 'className' parameters.
	 */
	String[] tag() default {};

	/**
	 * Specifies whether the compiler should emit a warning (the default) if the annotation doesn't match any html element at all.
	 */
	boolean warnIfUnused() default true;

	/**
	 * If the parameter is true, the content is included if the field/method returns false (instead of when it returns true).
	 */
	boolean inverse() default false;
}