/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.annotation.flowControl;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import org.jabhts.annotation.global.NotSpecified;

/**
 * Loops over a html element for each element an {@code Iterator} returns. For each element,
 * a scope is created (equivalent to {@link EScopeTag}), similar to how {@link EScopingWhile} does.
 * <p/>
 * Using the template:
 * <p/>
 * {@code <ul>}<br>
 * &nbsp; {@code <li id=NameRow><span id=Username>johndoe</span></li>}<br>
 * {@code </ul>}<p>
 * and the code:<p>
 * <code>List<String> usernames = ...;</code><br>
 * <code>@Iterate Iterable<String> iterNameRow() { return usernames; }</code><br>
 * <code>class EmitNameRow {</code><br>
 * &nbsp; <code>final String username;</code><br>
 * &nbsp; <code>EmitNameRow(String&nbsp;username) { this.username = username; }</code><br>
 * &nbsp; <code> @EText getUsername() { return username; }</code><br>
 * <code>}</code>
 * <p/>
 * the output is:<br>
 * {@code <ul>}<br>
 * &nbsp; {@code <li><span>hansen</span></li>}<br>
 * &nbsp; {@code <li><span>jensen</span></li>}<br>
 * &nbsp; {@code <li><span>frederiksen</span></li>}<br>
 * &nbsp; {@code <li><span>sørensen</span></li>}<br>
 * {@code </ul>}<p>
 * The instance of the inner-class is by default created by reflection. An instance-creating method can be
 * specified. Its name must be the name of the inner-class prepended with 'new', e.g.:<br>
 * <code>EmitNameRow newEmitNameRow(String x) { return new EmitNameRow(...); }</code>
 * <p>
 * The annotation must be applied to a field or method which returns an {@code Iterator<X>} or an {@code Iterable<X>}.
 * To avoid multiple declaration of the same id,
 * {@code id} attributes from the template are suppressed inside the loop
 *
 * @see EScopingWhile
 * @see EScopeTag
 */
@Target({ElementType.METHOD, ElementType.FIELD})
@Retention(RetentionPolicy.RUNTIME)
public @interface EIterate
{
	/**
	 * Makes the annotation match html elements which contains the value in the 'id' attribute or as a name in the 'class' attribute.
	 * Leading underscores in the names specified in the html are ignored when comparing the names.
	 * <p/>
	 * When the parameter is specified, the 'id', 'className' or 'tag' parameters must not be specified.
	 */
	String[] value() default {};

	/**
	 * Makes the annotation match html elements which contains the value in the 'id' attribute.
	 * Leading underscores in the 'id' attribute value are ignored when comparing the names.
	 * <p/>
	 * When the parameter is specified, the 'value' parameter must not be specified. It is allowed to specify
	 * the 'className' and 'tag' parameters.
	 */
	String[] id() default {};

	/**
	 * Makes the annotation match html elements which contains the value as a name in the 'class' attribute.
	 * Leading underscores in the names specified in the html are ignored when comparing the names.
	 * <p/>
	 * When the parameter is specified, the 'value' parameter must not be specified. It is allowed to specify
	 * the 'id' and 'tag' parameters.
	 */
	String[] className() default {};

	/**
	 * Makes the annotation match html elements whose tag is the specified value.
	 * <p/>
	 * When the parameter is specified, the 'value' parameter must not be specified. It is allowed to specify
	 * the 'id' and 'className' parameters.
	 */
	String[] tag() default {};

	/**
	 * Specifies whether the compiler should emit a warning (the default) if the annotation doesn't match any html element at all.
	 */
	boolean warnIfUnused() default true;

	/**
	 * The scope used for each iteration.
	 * It must be a class at the same (or higher) level as the method/field which @EIterate is applied to.
	 * <p/>
	 * If not specified, the inner-class is found by name which the name of the method/field
	 * (stripped by a lowercase prefix)
	 * prepended with "Emit". Thus, applied as {@code @EIterate List<T> iterPrinterRow() { ... }}
	 * it defaults to searching for {@code class EmitPrinterRow { ... } }
	 * <p/>
	 * The scope-class must have a constructor which takes one argument, which is the object the iterator returns.
	 */
	// todo: add link to page describing scope-instance-creation-method.
	Class<?> scope() default NotSpecified.class;
}
