/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.annotation.global;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Applying this annotation triggers compilation of a template. It can be applied to an outer or an inner-class.
 * If it is applied to an inner-class, the inner-class is used as a scope for the outer-class.
 * <p/>
 * The default filename for the html template is the name of the class: see
 * {@link #filename()}. It must be located in the same directory as the .java source file,
 * or in a sub-directory named "templates".
 * <p/>
 * As default, the complete html file is used (i.e. a normal html page). Fragments can be used by
 * using the {@link #fragment()} and {@link #fragmentChilds()} parameters.
 * <p/>
 * The name generated adapter class is the name of the class (which the annotation is applied to) suffixed by "Tp".
 * The name can be specified using the {@link #tpname()} parameter to
 * avoid multiple adapters being generated with the same name.
 * The adapter is generated in the same package as the outer-class. If a sub-directory "templatedata" exists, it is
 * put into that sub-package.
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.TYPE)
public @interface TCompile
{
	/**
	 * Filename (without suffix) of the html file.
	 * The default value (indicated by the empty string) is the name of the class which the annotation
	 * is applied to.
	 * <p/>
	 * <p style='margin-bottom: 0'>Valid suffixes:</p>
	 * <ul style='margin-top: 0'>
	 * <li>.html
	 * <li>.htm
	 * <li>.xhtml
	 * </ul>
	 * <p/>
	 * Html files are loaded using <a href='http://code.google.com/p/html5lib/'>html5lib</a>
	 * <p/>
	 * XHtml files are loaded using Java's XML api. The xhtml document is not validated.
	 */
	String filename() default "";

	/**
	 * Used when only a sub-section of the html document should be used instead of the complete page.
	 * Usefull for generating html snippets for e.g. Javascript's innerHTML.
	 * <p/>
	 * If empty: The complete document is output as a normal complete document.
	 * <br>
	 * Otherwise: finds the element whose id is the specified value, and
	 * uses the subtree.
	 *
	 * @see #fragmentChilds()
	 */
	String fragment() default "";

	/**
	 * Controls if the output includes the full document/subtree or only the body-childs/fragment-childs.
	 * <p/>
	 * If fragment() is the empty string: outputs only the childs of the <tt>&lt;body&gt;</tt> node.<br>
	 * <br>
	 * Otherwise:
	 * omits the tag for the element specified by fragment() and
	 * outputs only the elements childs
	 *
	 * @see #fragment()
	 */
	boolean fragmentChilds() default false;

	/**
	 * Classname of the generated adapter class.
	 * Must not include package name.
	 * <p/>
	 * The default value is the name of the class of the class (which the annotation is applied to) suffixed with "Tp".
	 * E.g. for a class named "StatusReport", the default name is "StatusReportTp".
	 * <p/>
	 * Even though the generated file is an adapter rather than a template, the default suffix is "Tp"
	 * instead of e.g. "TpAd" or "TA".
	 * <p/>
	 * The value (if specified) should normally include the "Tp" suffix.
	 */
	String tpname() default "";
}
