/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.translator.compiler.application.baseclass;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Arrays;
import org.jabhts.annotation.global.NotSpecified;
import org.jabhts.translator.compiler.application.scope.Scope;
import org.jabhts.translator.compiler.application.scope.SelectorType;
import org.jabhts.translator.compiler.application.scope.TargetingInfo;
import org.jabhts.translator.compiler.tree.ElementEmitters;
import org.jabhts.translator.compiler.utils.CompilerException;
import org.jabhts.translator.compiler.utils.CompilerLog;
import org.jabhts.translator.compiler.utils.CompilerUtils;
import org.jabhts.translator.compiler.utils.LogLevel;
import org.w3c.dom.Element;

/**
 * From "The Java Programming Language, Fourth Edition", page 388:
 * "Annotations are applied to program elements [...]
 * When you annotate (apply an annotation to) a program element ..."
 * <p/>
 * The annotation-as-class is called "Annotation", but when the annotation is used
 * (an instance in the source code), it is too called "annotation".
 * <p/>
 * Therefore, the annotation-instance is called "application" in the compiler, and the
 * annotation-class is "annotation".
 */
public abstract class Application
{
	public final CompilerLog log;
	//
	public final Scope scope;
	public final Annotation annotation;
	public final TargetingInfo target; // null if not targeting handler
	//
	// item which the annotation is specified on. Exactly one is non-null.
	public final Class<?> cls;
	public final Field field;
	public final Method method;

	protected Application(TargetingInfo target)
	{
		this.log = target.log;
		this.scope = target.scope;
		this.annotation = target.annotation;
		this.cls = target.cls;
		this.field = target.field;
		this.method = target.method;
		this.target = target;
	}

	public Scope makeSubScope(Scope outerScope) throws CompilerException
	{
		return null;
	}

	public abstract void createEmitter(ElementEmitters emitters, Element e) throws CompilerException;

	public String annotationToString()
	{
		return CompilerUtils.annotationToString(annotation, cls, field, method);
	}

	public String overrideKey()
	{
		StringBuilder sb = new StringBuilder(20);
		Class<?> annotClass = annotation.annotationType();
		sb.append(target.basename).append(" @ ").append(annotClass.getSimpleName());
		for (Method m : annotClass.getDeclaredMethods()) {
			String n = m.getName();
			if (n.equals("value") || n.equals("id") || n.equals("className") || n.equals("tag"))
				continue;
			if (n.equals("warnIfUnused"))
				continue;
			String value = getAnnotValue(m);
			if (value != null)
				sb.append(';').append(n).append("={<").append(value).append(">}");
		}
		log.log(LogLevel.Verbose, "OverrideKey: '" + sb + "'            for " + annotationToString());
		return sb.toString();
	}

	private String getAnnotValue(Method m)
	{
		try {
			Object obj = m.invoke(annotation);
			if (obj instanceof Boolean)
				return (Boolean)obj ? "true" : null;
			if (obj instanceof String) {
				String s = (String)obj;
				return (s.isEmpty()) ? null : s;
			}
			if (obj == NotSpecified.class)
				return null;
			if (obj instanceof String[])
				return Arrays.toString((String[])obj);
			else if (obj instanceof Boolean[])
				return Arrays.toString((Boolean[])obj);
			else if (obj instanceof Byte[])
				return Arrays.toString((Byte[])obj);
			else if (obj instanceof Integer[])
				return Arrays.toString((Integer[])obj);
			else if (obj instanceof Long[])
				return Arrays.toString((Long[])obj);
			else if (obj instanceof Float[])
				return Arrays.toString((Float[])obj);
			else if (obj instanceof Double[])
				return Arrays.toString((Double[])obj);
			else
				return obj.toString();
		}
		catch (Exception ex) {
			return "Exception=" + CompilerUtils.getSomeMessage(ex);
		}
	}

	@SuppressWarnings({"ThrowableInstanceNeverThrown"})
	public InvalidApplicationException makeInvalidAnnotationApplianceException(String msg)
	{
		return new InvalidApplicationException(msg + ": " + annotationToString());
	}

	public void notifyMatched(SelectorType type)
	{
		if (target != null)
			target.notifyMatched(type);
	}

	public void emitPostCompilationWarnings()
	{
		if (target != null)
			target.emitPostCompilationWarnings();
	}
}
